package com.diagbot.entity;

import cn.afterturn.easypoi.excel.annotation.Excel;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.diagbot.util.StringUtil;

import javax.validation.constraints.NotBlank;
import java.io.Serializable;
import java.util.Date;
import java.util.Objects;

/**
 * <p>
 * 中医证候映射表
 * </p>
 *
 * @author zhaops
 * @since 2021-05-13
 */
@TableName("tran_tcmsyndrome_config")
public class TcmsyndromeConfig implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 主键
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /**
     * 是否删除,N:未删除，Y:删除
     */
    private String isDeleted;

    /**
     * 记录创建时间
     */
    private Date gmtCreate;

    /**
     * 记录修改时间,如果时间是1970年则表示纪录未修改
     */
    private Date gmtModified;

    /**
     * 创建人，0表示无创建人值
     */
    private String creator;

    /**
     * 修改人,如果为0则表示纪录未修改
     */
    private String modifier;

    /**
     * 医院id
     */
    private Long hospitalId;

    /**
     * 医院项目名称
     */
    @Excel(name = "医院中医证候名称", width = 40, orderNum = "1", isImportField = "true")
    @NotBlank(message = "请输入医院中医证候名称")
    private String hisName;

    /**
     * 标准名称
     */
    @Excel(name = "标准中医证候名称", width = 40, orderNum = "3", isImportField = "true")
    @NotBlank(message = "请输入标准中医证候名称")
    private String uniqueName;

    /**
     * 标准编码
     */
    @Excel(name = "中医证候代码", width = 40, orderNum = "2", isImportField = "true")
    private String uniqueCode;

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getIsDeleted() {
        return isDeleted;
    }

    public void setIsDeleted(String isDeleted) {
        this.isDeleted = isDeleted;
    }

    public Date getGmtCreate() {
        return gmtCreate;
    }

    public void setGmtCreate(Date gmtCreate) {
        this.gmtCreate = gmtCreate;
    }

    public Date getGmtModified() {
        return gmtModified;
    }

    public void setGmtModified(Date gmtModified) {
        this.gmtModified = gmtModified;
    }

    public String getCreator() {
        return creator;
    }

    public void setCreator(String creator) {
        this.creator = creator;
    }

    public String getModifier() {
        return modifier;
    }

    public void setModifier(String modifier) {
        this.modifier = modifier;
    }

    public Long getHospitalId() {
        return hospitalId;
    }

    public void setHospitalId(Long hospitalId) {
        this.hospitalId = hospitalId;
    }

    public String getHisName() {
        return hisName;
    }

    public void setHisName(String hisName) {
        this.hisName = hisName;
    }

    public String getUniqueName() {
        return uniqueName;
    }

    public void setUniqueName(String uniqueName) {
        this.uniqueName = uniqueName;
    }

    public String getUniqueCode() {
        return uniqueCode;
    }

    public void setUniqueCode(String uniqueCode) {
        this.uniqueCode = uniqueCode;
    }

    @Override
    public String toString() {
        return "TcmsyndromeConfig{" +
                "id=" + id +
                ", isDeleted=" + isDeleted +
                ", gmtCreate=" + gmtCreate +
                ", gmtModified=" + gmtModified +
                ", creator=" + creator +
                ", modifier=" + modifier +
                ", hospitalId=" + hospitalId +
                ", hisName=" + hisName +
                ", uniqueName=" + uniqueName +
                ", uniqueCode=" + uniqueCode +
                "}";
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        TcmsyndromeConfig tcmsyndromeConfig = (TcmsyndromeConfig) o;
        return Objects.equals(id, tcmsyndromeConfig.id)
                && Objects.equals(isDeleted, tcmsyndromeConfig.isDeleted)
                && Objects.equals(hospitalId, tcmsyndromeConfig.hospitalId)
                && Objects.equals(hisName, tcmsyndromeConfig.hisName)
                && Objects.equals(uniqueName, tcmsyndromeConfig.uniqueName)
                && Objects.equals(uniqueCode, tcmsyndromeConfig.uniqueCode);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, isDeleted, hospitalId, hisName, uniqueName, uniqueCode);
    }

    public static boolean nonNull(TcmsyndromeConfig o) {
        return !(o == null
                || (o.hospitalId == null
                && StringUtil.isBlank(o.hisName)
                && StringUtil.isBlank(o.uniqueName)
                && StringUtil.isBlank(o.uniqueCode)));
    }
}