package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.IndexBatchDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.TcmsyndromeInfoDTO;
import com.diagbot.entity.TcmsyndromeConfig;
import com.diagbot.enums.ConceptTypeEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.TcmsyndromeConfigService;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ExcelUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.SysUserUtils;
import com.diagbot.vo.ConceptVO;
import com.diagbot.vo.IdListVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.RetrievalVO;
import com.diagbot.vo.TcmsyndromeConfigListVO;
import com.diagbot.vo.TcmsyndromeConfigPageVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/5/13 15:07
 */
@Component
public class TcmsyndromeConfigFacade {
    @Autowired
    private TcmsyndromeConfigService tcmsyndromeConfigService;
    @Autowired
    private CdssCoreClient cdssCoreClient;

    /**
     * 判断是否已存在
     *
     * @param tcmsyndromeConfig
     * @return
     */
    public Boolean isExistRecord(TcmsyndromeConfig tcmsyndromeConfig) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        QueryWrapper<TcmsyndromeConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", Long.valueOf(hospitalId))
                .eq("his_name", tcmsyndromeConfig.getHisName())
                .eq("unique_name", tcmsyndromeConfig.getUniqueName());
        TcmsyndromeConfig oldRecord = tcmsyndromeConfigService.getOne(queryWrapper, false);
        if (tcmsyndromeConfig.getId() == null
                && oldRecord != null) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法添加");
        }
        if (tcmsyndromeConfig.getId() != null
                && oldRecord != null
                && !tcmsyndromeConfig.getId().equals(oldRecord.getId())) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        return false;
    }

    /**
     * 保存记录-单条
     *
     * @param tcmsyndromeConfig
     * @return
     */
    public Boolean saveOrUpdateRecord(TcmsyndromeConfig tcmsyndromeConfig) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
        tcmsyndromeConfig.setHospitalId(Long.valueOf(hospitalId));
        tcmsyndromeConfig.setModifier(userId);
        tcmsyndromeConfig.setGmtModified(now);
        QueryWrapper<TcmsyndromeConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", Long.valueOf(hospitalId))
                .eq("his_name", tcmsyndromeConfig.getHisName())
                .eq("unique_name", tcmsyndromeConfig.getUniqueName());
        TcmsyndromeConfig oldRecord = tcmsyndromeConfigService.getOne(queryWrapper, false);
        if (tcmsyndromeConfig.getId() == null
                && oldRecord != null) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法添加");
        }
        if (tcmsyndromeConfig.getId() != null
                && oldRecord != null
                && !tcmsyndromeConfig.getId().equals(oldRecord.getId())) {
            throw new CommonException(CommonErrorCode.IS_EXISTS, "该条关联已存在，无法保存");
        }
        //新增数据
        if (tcmsyndromeConfig.getId() == null) {
            tcmsyndromeConfig.setCreator(userId);
            tcmsyndromeConfig.setGmtCreate(now);
        }
        if (tcmsyndromeConfig.getIsDeleted() == null) {
            tcmsyndromeConfig.setIsDeleted(IsDeleteEnum.N.getKey());
        }
        tcmsyndromeConfigService.saveOrUpdate(tcmsyndromeConfig);
        return true;
    }

    /**
     * 保存记录-批量
     *
     * @param tcmsyndromeConfigListVO
     * @return
     */
    public Boolean saveOrUpdateRecords(TcmsyndromeConfigListVO tcmsyndromeConfigListVO) {
        if (ListUtil.isEmpty(tcmsyndromeConfigListVO.getTcmsyndromeConfigList())) {
            return false;
        }
        return saveOrUpdateRecords(tcmsyndromeConfigListVO.getTcmsyndromeConfigList());
    }

    /**
     * 批量保存
     *
     * @param tcmsyndromeConfigList
     * @return
     */
    public Boolean saveOrUpdateRecords(List<TcmsyndromeConfig> tcmsyndromeConfigList) {
        if (ListUtil.isEmpty(tcmsyndromeConfigList)) {
            return false;
        }
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();

        //数据不完整的不保存
        //过滤外部名称或公表名为空的数据
        tcmsyndromeConfigList = tcmsyndromeConfigList
                .stream()
                .filter(i -> StringUtil.isNotBlank(i.getHisName()))
                .filter(i -> StringUtil.isNotBlank(i.getUniqueName()))
                .collect(Collectors.toList());

        // 验证数据是否已存在，已存在的先删除
        // 没id的删除重新插入，有id的更新
        List<Long> deleteIds = Lists.newLinkedList();
        Map<String, Map<String, List<Long>>> configMap
                = getConfigMapWithIds(Long.valueOf(hospitalId), null, null);
        tcmsyndromeConfigList.forEach(tcmsyndromeConfig -> {
            tcmsyndromeConfig.setHospitalId(Long.valueOf(hospitalId));
            tcmsyndromeConfig.setModifier(userId);
            tcmsyndromeConfig.setGmtModified(now);
            if (tcmsyndromeConfig.getId() == null) {
                if (configMap.containsKey(tcmsyndromeConfig.getHisName())
                        && ListUtil.isNotEmpty(configMap.get(tcmsyndromeConfig.getHisName()).get(tcmsyndromeConfig.getUniqueName()))) {
                    deleteIds.addAll(configMap.get(tcmsyndromeConfig.getHisName()).get(tcmsyndromeConfig.getUniqueName()));
                }
                tcmsyndromeConfig.setCreator(userId);
                tcmsyndromeConfig.setGmtCreate(now);
            }
            if (tcmsyndromeConfig.getIsDeleted() == null) {
                tcmsyndromeConfig.setIsDeleted(IsDeleteEnum.N.getKey());
            }
        });
        //删除已存在映射关系
        IdListVO idListVO = new IdListVO();
        idListVO.setIds(deleteIds);
        deleteRecords(idListVO);
        tcmsyndromeConfigService.saveOrUpdateBatch(tcmsyndromeConfigList);
        return true;
    }

    /**
     * 删除记录-单条
     *
     * @param idVO
     * @return
     */
    public Boolean deleteRecord(IdVO idVO) {
        UpdateWrapper<TcmsyndromeConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", idVO.getId())
                .set("is_deleted", IsDeleteEnum.Y.getKey());
        tcmsyndromeConfigService.removeById(idVO.getId());
        return true;
    }

    /**
     * 删除记录-批量
     *
     * @param idListVO
     * @return
     */
    public Boolean deleteRecords(IdListVO idListVO) {
        if (ListUtil.isEmpty(idListVO.getIds())) {
            return false;
        }
        UpdateWrapper<TcmsyndromeConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.in("id", idListVO.getIds())
                .set("is_deleted", IsDeleteEnum.Y.getKey());
        tcmsyndromeConfigService.removeByIds(idListVO.getIds());
        return true;
    }

    /**
     * 分页查询
     *
     * @param tcmsyndromeConfigPageVO
     * @return
     */
    public IPage<TcmsyndromeConfig> getPage(TcmsyndromeConfigPageVO tcmsyndromeConfigPageVO) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        tcmsyndromeConfigPageVO.setHospitalId(Long.valueOf(hospitalId));
        return tcmsyndromeConfigService.getPage(tcmsyndromeConfigPageVO);
    }

    /**
     * 数据导入
     *
     * @param file
     */
    public void importExcel(MultipartFile file) {
        List<TcmsyndromeConfig> tcmsyndromeConfigList = ExcelUtils.importExcel(file, 0, 1, TcmsyndromeConfig.class);
        if (ListUtil.isNotEmpty(tcmsyndromeConfigList)) {
            importExcelRecords(tcmsyndromeConfigList);
        } else {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "校验失败，导入数据不能为空");
        }
    }

    /**
     * 数据导入
     *
     * @param tcmsyndromeConfigList
     * @return
     */
    public Boolean importExcelRecords(List<TcmsyndromeConfig> tcmsyndromeConfigList) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String userId = SysUserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();


        //1、数据完整性校验
        //2、去除前后空格
        //过滤空数据，保留重复数据，方便计行
        tcmsyndromeConfigList = tcmsyndromeConfigList.stream()
                .filter(TcmsyndromeConfig::nonNull)
                .collect(Collectors.toList());
        if (ListUtil.isEmpty(tcmsyndromeConfigList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "校验失败，导入数据不能为空");
        }
        List<String> emptyNumList = Lists.newLinkedList();
        for (int i = 0; i < tcmsyndromeConfigList.size(); i++) {
            if (StringUtil.isBlank(tcmsyndromeConfigList.get(i).getHisName())
                    || StringUtil.isBlank(tcmsyndromeConfigList.get(i).getUniqueName())) {
                emptyNumList.add(String.valueOf(i + 2));
            }
            if (StringUtil.isNotBlank(tcmsyndromeConfigList.get(i).getHisName())) {
                tcmsyndromeConfigList.get(i).setHisName(tcmsyndromeConfigList.get(i).getHisName().trim());
            }
            if (StringUtil.isNotBlank(tcmsyndromeConfigList.get(i).getUniqueName())) {
                tcmsyndromeConfigList.get(i).setUniqueName(tcmsyndromeConfigList.get(i).getUniqueName().trim());
            }
            if (StringUtil.isNotBlank(tcmsyndromeConfigList.get(i).getUniqueCode())) {
                tcmsyndromeConfigList.get(i).setUniqueCode(tcmsyndromeConfigList.get(i).getUniqueCode().trim());
            } else {
                tcmsyndromeConfigList.get(i).setUniqueCode(null);
            }
        }

        if (ListUtil.isNotEmpty(emptyNumList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "以下行数（不计入空行）存在不完整数据："
                    + emptyNumList.stream().collect(Collectors.joining("、"))
                    + "。导入取消，请修改后再试。\n");
        }


        // 验证数据是否已存在，已存在的先删除
        // 没id的删除重新插入，有id的更新
        synchronized (this) {
            List<Long> deleteIds = Lists.newLinkedList();
            Map<String, Map<String, List<Long>>> configMap
                    = getConfigMapWithIds(Long.valueOf(hospitalId), null, null);
            tcmsyndromeConfigList.forEach(tcmsyndromeConfig -> {
                tcmsyndromeConfig.setHospitalId(Long.valueOf(hospitalId));
                tcmsyndromeConfig.setModifier(userId);
                tcmsyndromeConfig.setGmtModified(now);
                if (tcmsyndromeConfig.getId() == null) {
                    if (configMap.containsKey(tcmsyndromeConfig.getHisName())
                            && ListUtil.isNotEmpty(configMap.get(tcmsyndromeConfig.getHisName()).get(tcmsyndromeConfig.getUniqueName()))) {
                        deleteIds.addAll(configMap.get(tcmsyndromeConfig.getHisName()).get(tcmsyndromeConfig.getUniqueName()));
                    }
                    tcmsyndromeConfig.setCreator(userId);
                    tcmsyndromeConfig.setGmtCreate(now);
                }
                if (tcmsyndromeConfig.getIsDeleted() == null) {
                    tcmsyndromeConfig.setIsDeleted(IsDeleteEnum.N.getKey());
                }
            });

            //标准术语校验
            List<String> errorNumList = Lists.newLinkedList();
            List<String> uniqueNames = tcmsyndromeConfigList.stream()
                    .map(i -> i.getUniqueName())
                    .distinct()
                    .collect(Collectors.toList());
            ConceptVO conceptVO = new ConceptVO();
            conceptVO.setNames(uniqueNames);
            conceptVO.setType(ConceptTypeEnum.Tcmsyndrome.getKey());
            RespDTO<List<IndexBatchDTO>> respDTO = cdssCoreClient.getConceptNames(conceptVO);
            RespDTOUtil.respNGDealCover(respDTO, "标准术语校验失败");
            List<String> names = respDTO.data.stream().map(IndexBatchDTO::getName).collect(Collectors.toList());
            for (int i = 0; i < tcmsyndromeConfigList.size(); i++) {
                if (!names.contains(tcmsyndromeConfigList.get(i).getUniqueName())) {
                    errorNumList.add(String.valueOf(i + 2));
                }
            }
            if (ListUtil.isNotEmpty(errorNumList)) {
                throw new CommonException(CommonErrorCode.PARAM_IS_NULL,
                        "以下行数（不计空行）标准术语在数据库中不存在："
                                + errorNumList.stream().collect(Collectors.joining("、"))
                                + "。导入取消，请修改后再试。");
            }

            //重复数据过滤
            tcmsyndromeConfigList = tcmsyndromeConfigList
                    .stream()
                    .distinct()
                    .collect(Collectors.toList());

            //删除已存在映射关系
            IdListVO idListVO = new IdListVO();
            idListVO.setIds(deleteIds);
            deleteRecords(idListVO);
            tcmsyndromeConfigService.saveOrUpdateBatch(tcmsyndromeConfigList);
        }
        return true;
    }

    /**
     * 获取映射关系-公表名
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return
     */
    public Map<String, Map<String, Long>> getConfigMap(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, Long>> retMap = new HashMap<>();
        QueryWrapper<TcmsyndromeConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<TcmsyndromeConfig> records = tcmsyndromeConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }
        Map<String, List<TcmsyndromeConfig>> configMap = EntityUtil.makeEntityListMap(records, "hisName");
        for (Map.Entry<String, List<TcmsyndromeConfig>> entry : configMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                retMap.put(entry.getKey(), EntityUtil.makeMapWithKeyValue(entry.getValue(), "uniqueName", "id"));
            }
        }
        return retMap;
    }

    /**
     * 获取映射关系-公表名
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return
     */
    public Map<String, Map<String, List<Long>>> getConfigMapWithIds(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, List<Long>>> retMap = new HashMap<>();
        QueryWrapper<TcmsyndromeConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<TcmsyndromeConfig> records = tcmsyndromeConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }
        Map<String, List<TcmsyndromeConfig>> configMap = EntityUtil.makeEntityListMap(records, "hisName");
        for (Map.Entry<String, List<TcmsyndromeConfig>> entry : configMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                Map<String, List<TcmsyndromeConfig>> subMap = EntityUtil.makeEntityListMap(entry.getValue(), "uniqueName");
                Map<String, List<Long>> subIdMap = new HashMap<>();
                for (Map.Entry<String, List<TcmsyndromeConfig>> subEntry : subMap.entrySet()) {
                    subIdMap.put(subEntry.getKey(), subEntry.getValue().stream().map(i -> i.getId()).distinct().collect(Collectors.toList()));
                }
                retMap.put(entry.getKey(), subIdMap);
            }
        }
        return retMap;
    }

    /**
     * 获取映射关系
     * Map<uniqueName,Map<hisName,id>>
     *
     * @param hospitalId
     * @param hisNames
     * @param uniqueNames
     * @return
     */
    public Map<String, Map<String, Long>> getUniqueNameConfigMap(Long hospitalId, List<String> hisNames, List<String> uniqueNames) {
        Map<String, Map<String, Long>> retMap = new HashMap<>();
        QueryWrapper<TcmsyndromeConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", hospitalId);
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        if (ListUtil.isNotEmpty(uniqueNames)) {
            queryWrapper.in("unique_name", uniqueNames);
        }
        List<TcmsyndromeConfig> records = tcmsyndromeConfigService.list(queryWrapper);
        if (ListUtil.isEmpty(records)) {
            return retMap;
        }

        Map<String, List<TcmsyndromeConfig>> uniqueNameMap = EntityUtil.makeEntityListMap(records, "uniqueName");
        for (Map.Entry<String, List<TcmsyndromeConfig>> entry : uniqueNameMap.entrySet()) {
            if (ListUtil.isNotEmpty(entry.getValue())) {
                retMap.put(entry.getKey(), EntityUtil.makeMapWithKeyValue(entry.getValue(), "hisName", "id"));
            }
        }
        return retMap;
    }

    /**
     * 数据导出
     *
     * @param response
     */
    public void exportExcel(HttpServletResponse response) {
        QueryWrapper<TcmsyndromeConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", SysUserUtils.getCurrentHospitalID())
                .orderByDesc("gmt_modified");
        List<TcmsyndromeConfig> records = tcmsyndromeConfigService.list(queryWrapper);
        String fileName = "中医证候映射.xls";
        ExcelUtils.exportExcel(records, null, "sheet1", TcmsyndromeConfig.class, fileName, response, 12.8f);
    }

    /**
     * 数据导入模板导出
     *
     * @param response
     */
    public void exportExcelModule(HttpServletResponse response) {
        String fileName = "中医证候映射模板.xls";
        ExcelUtils.exportExcel(new ArrayList<>(), null, "sheet1", TcmsyndromeConfig.class, fileName, response, 12.8f);
    }

    /**
     * 中医证候搜索
     *
     * @param retrievalVO
     * @return
     */
    public List<TcmsyndromeInfoDTO> getTcmsyndromes(RetrievalVO retrievalVO) {
        List<TcmsyndromeInfoDTO> tcmsyndromeNames = new ArrayList<>();
        List<TcmsyndromeConfig> tcmsyndromeConfigList = tcmsyndromeConfigService.getTcmsyndromesIndex(retrievalVO);
        for (TcmsyndromeConfig tcmsyndromeConfig : tcmsyndromeConfigList) {
            TcmsyndromeInfoDTO tcmsyndromeInfoDTO = new TcmsyndromeInfoDTO();
            tcmsyndromeInfoDTO.setCode(tcmsyndromeConfig.getUniqueCode());
            tcmsyndromeInfoDTO.setName(tcmsyndromeConfig.getHisName());
            tcmsyndromeNames.add(tcmsyndromeInfoDTO);
        }
        return tcmsyndromeNames;
    }
}