package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.NurseConfig;
import com.diagbot.facade.NurseConfigFacade;
import com.diagbot.vo.IdListVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.NurseConfigListVO;
import com.diagbot.vo.NurseConfigPageVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

/**
 * <p>
 * 护理映射表 前端控制器
 * </p>
 *
 * @author zhaops
 * @since 2021-04-26
 */
@RestController
@RequestMapping("/tran/nurseConfig")
@Api(value = "护理公表映射API", tags = { "护理公表映射API" })
@SuppressWarnings("unchecked")
public class NurseConfigController {
    @Autowired
    private NurseConfigFacade nurseConfigFacade;

    /**
     * 映射关系是否已存在
     *
     * @param nurseConfig
     * @return
     */
    @ApiOperation(value = "映射关系是否已存在[by:zhaops]", notes = "")
    @PostMapping("/isExistRecord")
    @SysLogger("isExistRecord")
    public RespDTO<Boolean> isExistRecord(@RequestBody @Valid NurseConfig nurseConfig) {
        Boolean data = nurseConfigFacade.isExistRecord(nurseConfig);
        return RespDTO.onSuc(data);
    }

    /**
     * 保存或修改映射关系
     *
     * @param nurseConfig
     * @return
     */
    @ApiOperation(value = "保存或修改映射关系[by:zhaops]", notes = "")
    @PostMapping("/saveOrUpdateRecord")
    @SysLogger("saveOrUpdateRecord")
    @Transactional
    public RespDTO<Boolean> saveOrUpdateRecord(@RequestBody @Valid NurseConfig nurseConfig) {
        Boolean data = nurseConfigFacade.saveOrUpdateRecord(nurseConfig);
        return RespDTO.onSuc(data);
    }

    /**
     * 批量保存或修改映射关系
     *
     * @param nurseConfigListVO
     * @return
     */
    @ApiOperation(value = "批量保存或修改映射关系[by:zhaops]", notes = "")
    @PostMapping("/saveOrUpdateRecords")
    @SysLogger("saveOrUpdateRecords")
    @Transactional
    public RespDTO<Boolean> saveOrUpdateRecords(@RequestBody @Valid NurseConfigListVO nurseConfigListVO) {
        Boolean data = nurseConfigFacade.saveOrUpdateRecords(nurseConfigListVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 删除映射关系
     *
     * @param idVO
     * @return
     */
    @ApiOperation(value = "删除映射关系[by:zhaops]", notes = "")
    @PostMapping("/deleteRecord")
    @SysLogger("deleteRecord")
    @Transactional
    public RespDTO<Boolean> deleteRecord(@RequestBody @Valid IdVO idVO) {
        Boolean data = nurseConfigFacade.deleteRecord(idVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 批量删除映射关系
     *
     * @param idListVO
     * @return
     */
    @ApiOperation(value = "批量删除映射关系[by:zhaops]", notes = "")
    @PostMapping("/deleteRecords")
    @SysLogger("deleteRecords")
    @Transactional
    public RespDTO<Boolean> deleteRecords(@RequestBody @Valid IdListVO idListVO) {
        Boolean data = nurseConfigFacade.deleteRecords(idListVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 分页查询
     *
     * @param nurseConfigPageVO
     * @return
     */
    @ApiOperation(value = "分页查询[by:zhaops]", notes = "")
    @PostMapping("/getPage")
    @SysLogger("getPage")
    public RespDTO<NurseConfig> getPage(@RequestBody @Valid NurseConfigPageVO nurseConfigPageVO) {
        IPage<NurseConfig> data = nurseConfigFacade.getPage(nurseConfigPageVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 公表数据导入
     *
     * @param file
     * @return
     */
    @ApiOperation(value = "公表数据导入[by:zhaops]",
            notes = "")
    @PostMapping("/importExcel")
    @SysLogger("importExcel")
    @Transactional
    public void importExcel(@RequestParam("file") MultipartFile file) {
        nurseConfigFacade.importExcel(file);
    }

    /**
     * 数据导出
     *
     * @return
     */
    @ApiOperation(value = "数据导出[by:gaodm]",
            notes = "")
    @PostMapping("/exportExcel")
    @SysLogger("exportExcel")
    public void exportExcel(HttpServletResponse response) {
        nurseConfigFacade.exportExcel(response);
    }

    /**
     * 数据导入模板导出
     *
     * @return
     */
    @ApiOperation(value = "数据导入模板导出[by:zhaops]",
            notes = "")
    @PostMapping("/exportExcelModule")
    @SysLogger("exportExcelModule")
    public void exportExcelModule(HttpServletResponse response) {
        nurseConfigFacade.exportExcelModule(response);
    }
}
